@@ -30,8 +30,11 @@ then
    shift
 fi
 
+luashortversion=`echo $luaversion | cut -d. -f 1-2`
+
 testing_dir="$PWD"
 
+testing_lrprefix="$testing_dir/testing_lrprefix-$luaversion"
 testing_tree="$testing_dir/testing-$luaversion"
 testing_sys_tree="$testing_dir/testing_sys-$luaversion"
 testing_tree_copy="$testing_dir/testing_copy-$luaversion"
@@ -39,7 +42,6 @@ testing_sys_tree_copy="$testing_dir/testing_sys_copy-$luaversion"
 testing_cache="$testing_dir/testing_cache-$luaversion"
 testing_server="$testing_dir/testing_server-$luaversion"
 
-
 if [ "$1" == "--clean" ]
 then
    shift
@@ -51,6 +53,7 @@ fi
 rm -f luacov.report.out
 rm -rf /tmp/luarocks_testing
 mkdir /tmp/luarocks_testing
+rm -rf "$testing_lrprefix"
 rm -rf "$testing_tree"
 rm -rf "$testing_sys_tree"
 rm -rf "$testing_tree_copy"
@@ -77,7 +80,7 @@ rocks_servers = {
 }
 local_cache = "$testing_cache"
 upload_server = "testing"
-upload_user = "hisham"
+upload_user = "$USER"
 upload_servers = {
    testing = {
       rsync = "localhost/tmp/luarocks_testing",
@@ -107,7 +110,7 @@ rocks_trees = {
 }
 local_cache = "$testing_cache"
 upload_server = "testing"
-upload_user = "hisham"
+upload_user = "$USER"
 upload_servers = {
    testing = {
       sftp = "localhost/tmp/luarocks_testing",
@@ -130,6 +133,9 @@ return {
     "luacov%.stats$",
     "luacov%.tick$",
   },
+--  ["coveralls"] = {
+--     ["pathcorrect"] = {"..."},
+--  },
 }
 EOF
 
@@ -153,8 +159,16 @@ then
       make install INSTALL_TOP="$luadir" &> /dev/null
    fi
    popd
+   [ -e ~/.ssh/id_rsa.pub ] || ssh-keygen -t rsa -P "" -f ~/.ssh/id_rsa
+   cat ~/.ssh/id_rsa.pub >> ~/.ssh/authorized_keys
+   chmod og-wx ~/.ssh/authorized_keys
+   ssh-keyscan localhost >> ~/.ssh/known_hosts
 else
    luadir="/Programs/Lua/Current"
+   if [ ! -e "$luadir" ]
+   then
+      luadir="/usr/local"
+   fi
 fi
 
 if [ `uname -m` = i686 ]
@@ -171,19 +185,21 @@ verrev_luasocket=${version_luasocket}-1
 srcdir_luasocket=luasocket-3.0-rc1
 
 version_cprint=0.1
-verrev_cprint=0.1-1
+verrev_cprint=0.1-2
 
-version_luacov=0.5
-verrev_luacov=0.5-1
+version_luacov=0.7
+verrev_luacov=${version_luacov}-1
 version_lxsh=0.8.6
 version_validate_args=1.5.4
 verrev_validate_args=1.5.4-1
 verrev_lxsh=${version_lxsh}-2
+version_abelhas=1.0
+verrev_abelhas=${version_abelhas}-1
 
 luasec=luasec
 
 cd ..
-./configure --with-lua="$luadir"
+./configure --with-lua="$luadir" --prefix="$testing_lrprefix"
 make clean
 make src/luarocks/site_config.lua
 make dev
@@ -217,8 +233,7 @@ luarocks_admin_nocov="run_lua --nocov luarocks-admin"
 mkdir -p "$testing_server"
 (
    cd "$testing_server"
-   luarocks_repo="http://luarocks.org/repositories/rocks"
-   luarocks_scm_repo="http://luarocks.org/repositories/rocks-scm"
+   luarocks_repo="http://rocks.moonscript.org"
    get() { [ -e `basename "$1"` ] || wget -c "$1"; }
    get "$luarocks_repo/luacov-${verrev_luacov}.src.rock"
    get "$luarocks_repo/luacov-${verrev_luacov}.rockspec"
@@ -226,26 +241,31 @@ mkdir -p "$testing_server"
    get "$luarocks_repo/lualogging-1.3.0-1.src.rock"
    get "$luarocks_repo/luasocket-${verrev_luasocket}.src.rock"
    get "$luarocks_repo/luasocket-${verrev_luasocket}.rockspec"
-   get "$luarocks_repo/luafilesystem-1.6.2-1.src.rock"
-   get "$luarocks_repo/stdlib-35-1.src.rock"
+   get "$luarocks_repo/luafilesystem-1.6.3-1.src.rock"
+   get "$luarocks_repo/stdlib-41.0.0-1.src.rock"
    get "$luarocks_repo/luarepl-0.4-1.src.rock"
    get "$luarocks_repo/validate-args-1.5.4-1.rockspec"
-   get "$luarocks_scm_repo/luasec-scm-1.rockspec"
+   get "$luarocks_repo/luasec-0.5-2.rockspec"
    get "$luarocks_repo/luabitop-1.0.2-1.rockspec"
    get "$luarocks_repo/lpty-1.0.1-1.src.rock"
    get "$luarocks_repo/cprint-${verrev_cprint}.src.rock"
    get "$luarocks_repo/cprint-${verrev_cprint}.rockspec"
    get "$luarocks_repo/wsapi-1.6-1.src.rock"
    get "$luarocks_repo/lxsh-${verrev_lxsh}.src.rock"
-   get "$luarocks_repo/abelhas-1.0-1.rockspec"
-   get "$luarocks_repo/lzlib-0.4.work3-1.src.rock"
+   get "$luarocks_repo/lxsh-${verrev_lxsh}.rockspec"
+   get "$luarocks_repo/abelhas-${verrev_abelhas}.rockspec"
+   get "$luarocks_repo/lzlib-0.4.1.53-1.src.rock"
    get "$luarocks_repo/lpeg-0.12-1.src.rock"
-   get "$luarocks_repo/luaposix-31-1.src.rock"
+   get "$luarocks_repo/luaposix-33.2.1-1.src.rock"
    get "$luarocks_repo/md5-1.2-1.src.rock"
-   get "$luarocks_repo/lrandom-20120430.51-1.src.rock"
-   get "$luarocks_repo/lrandom-20120430.52-1.src.rock"
-   get "$luarocks_repo/lrandom-20120430.51-1.rockspec"
-   get "$luarocks_repo/lrandom-20120430.52-1.rockspec"
+   get "$luarocks_repo/lmathx-20120430.51-1.src.rock"
+   get "$luarocks_repo/lmathx-20120430.52-1.src.rock"
+   get "$luarocks_repo/lmathx-20120430.51-1.rockspec"
+   get "$luarocks_repo/lmathx-20120430.52-1.rockspec"
+   get "$luarocks_repo/lmathx-20140620-1.rockspec"
+   get "$luarocks_repo/lua-path-0.2.3-1.src.rock"
+   get "$luarocks_repo/lua-cjson-2.1.0-1.src.rock"
+   get "$luarocks_repo/luacov-coveralls-0.1.1-1.src.rock"
 )
 $luarocks_admin_nocov make_manifest "$testing_server"
 
@@ -270,6 +290,8 @@ build_environment() {
          $luarocks_nocov pack --tree="$testing_sys_tree" $package; mv $package-*.rock "$testing_cache"
       }
    done
+   export LUA_PATH=
+   export LUA_CPATH=
    eval `$luarocks_noecho_nocov path --bin`
    cp -a "$testing_tree" "$testing_tree_copy"
    cp -a "$testing_sys_tree" "$testing_sys_tree_copy"
@@ -316,12 +338,19 @@ need_luasocket() { need luasocket $verrev_luasocket; }
 
 test_version() { $luarocks --version; }
 
-fail_arg_server() { $luarocks --server; }
-fail_arg_only_server() { $luarocks --only-server; }
 fail_unknown_command() { $luarocks unknown_command; }
 
+fail_arg_boolean_parameter() { $luarocks --porcelain=invalid; }
+fail_arg_boolean_unknown() { $luarocks --invalid-flag; }
+fail_arg_string_no_parameter() { $luarocks --server; }
+fail_arg_string_followed_by_flag() { $luarocks --server --porcelain; }
+fail_arg_string_unknown() { $luarocks --invalid-flag=abc; }
+
 test_empty_list() { $luarocks list; }
 
+fail_sysconfig_err() { local err=0; local scdir="$testing_lrprefix/etc/luarocks/"; mkdir -p "$scdir"; local sysconfig="$scdir/config.lua"; echo "aoeui" > "$sysconfig"; echo $sysconfig; $luarocks list; err=$?; rm "$sysconfig"; return "$err"; }
+fail_sysconfig_default_err() { local err=0; local scdir="$testing_lrprefix/etc/luarocks/"; mkdir -p "$scdir"; local sysconfig="$scdir/config-$luashortversion.lua"; echo "aoeui" > "$sysconfig"; echo $sysconfig; $luarocks list; err=$?; rm "$sysconfig"; return "$err"; }
+
 fail_build_noarg() { $luarocks build; }
 fail_download_noarg() { $luarocks download; }
 fail_install_noarg() { $luarocks install; }
@@ -329,6 +358,7 @@ fail_lint_noarg() { $luarocks lint; }
 fail_search_noarg() { $luarocks search; }
 fail_show_noarg() { $luarocks show; }
 fail_unpack_noarg() { $luarocks unpack; }
+fail_upload_noarg() { $luarocks upload; }
 fail_remove_noarg() { $luarocks remove; }
 fail_doc_noarg() { $luarocks doc; }
 fail_new_version_noarg() { $luarocks new_version; }
@@ -341,8 +371,16 @@ fail_lint_invalid() { $luarocks lint invalid; }
 fail_show_invalid() { $luarocks show invalid; }
 fail_new_version_invalid() { $luarocks new_version invalid; }
 
+test_list_invalidtree() { $luarocks --tree=/some/invalid/tree list; }
+
+fail_inexistent_dir() { mkdir idontexist; cd idontexist; rmdir ../idontexist; $luarocks; err=$?; cd ..; return $err; }
+
 fail_make_norockspec() { $luarocks make; }
 
+fail_build_permissions() { $luarocks build --tree=/usr lpeg; }
+fail_build_permissions_parent() { $luarocks build --tree=/usr/invalid lpeg; }
+
+test_build_verbose() { $luarocks build --verbose lpeg; }
 fail_build_blank_arg() { $luarocks build --tree="" lpeg; }
 test_build_withpatch() { need_luasocket; $luarocks build luadoc; }
 test_build_diffversion() { $luarocks build luacov ${version_luacov}; }
@@ -351,19 +389,33 @@ test_build_install_bin() { $luarocks build luarepl; }
 test_build_nohttps() { need_luasocket; $luarocks download --rockspec validate-args ${verrev_validate_args} && $luarocks build ./validate-args-${version_validate_args}-1.rockspec && rm ./validate-args-${version_validate_args}-1.rockspec; }
 test_build_https() { need_luasocket; $luarocks download --rockspec validate-args ${verrev_validate_args} && $luarocks install $luasec && $luarocks build ./validate-args-${verrev_validate_args}.rockspec && rm ./validate-args-${verrev_validate_args}.rockspec; }
 test_build_supported_platforms() { $luarocks build lpty; }
+test_build_only_deps_rockspec() { $luarocks download --rockspec lxsh ${verrev_lxsh} && $luarocks build ./lxsh-${verrev_lxsh}.rockspec --only-deps && { $luarocks show lxsh; [ $? -ne 0 ]; }; }
+test_build_only_deps_src_rock() { $luarocks download --source lxsh ${verrev_lxsh} && $luarocks build ./lxsh-${verrev_lxsh}.src.rock --only-deps && { $luarocks show lxsh; [ $? -ne 0 ]; }; }
+test_build_only_deps() { $luarocks build luasec --only-deps && { $luarocks show luasec; [ $? -ne 0 ]; }; }
+test_install_only_deps() { $luarocks install lxsh ${verrev_lxsh} --only-deps && { $luarocks show lxsh; [ $? -ne 0 ]; }; }
 fail_build_missing_external() { $luarocks build "$testing_dir/testfiles/missing_external-0.1-1.rockspec" INEXISTENT_INCDIR="/invalid/dir"; }
+fail_build_invalidpatch() { need_luasocket; $luarocks build "$testing_dir/testfiles/invalid_patch-0.1-1.rockspec"; }
 
-test_build_deps_partial_match() { $luarocks build lrandom; }
+test_build_deps_partial_match() { $luarocks build lmathx; }
 test_build_show_downloads() { export LUAROCKS_CONFIG="$testing_dir/testing_config_show_downloads.lua" && $luarocks build alien; export LUAROCKS_CONFIG="$testing_dir/testing_config.lua"; }
 
 test_download_all() { $luarocks download --all validate-args && rm validate-args-*; }
 test_download_rockspecversion() { $luarocks download --rockspec validate-args ${verrev_validate_args} && rm validate-args-*; }
 
 test_help() { $luarocks help; }
+fail_help_invalid() { $luarocks help invalid; }
 
 test_install_binaryrock() { $luarocks build --pack-binary-rock cprint && $luarocks install ./cprint-${verrev_cprint}.${platform}.rock && rm ./cprint-${verrev_cprint}.${platform}.rock; }
 test_install_with_bin() { $luarocks install wsapi; }
 fail_install_notazipfile() { $luarocks install "$testing_dir/testfiles/not_a_zipfile-1.0-1.src.rock"; }
+fail_install_invalidpatch() { need_luasocket; $luarocks install "$testing_dir/testfiles/invalid_patch-0.1-1.rockspec"; }
+fail_install_invalid_filename() { $luarocks install "invalid.rock"; }
+fail_install_invalid_arch() { $luarocks install "foo-1.0-1.impossible-x86.rock"; }
+test_install_reinstall() { $luarocks install "$testing_cache/luasocket-$verrev_luasocket.$platform.rock"; $luarocks install --deps-mode=none "$testing_cache/luasocket-$verrev_luasocket.$platform.rock"; }
+
+fail_local_root() { USER=root $luarocks install --local luasocket; }
+
+test_site_config() { mv ../src/luarocks/site_config.lua ../src/luarocks/site_config.lua.tmp; $luarocks; mv ../src/luarocks/site_config.lua.tmp ../src/luarocks/site_config.lua; }
 
 test_lint_ok() { $luarocks download --rockspec validate-args ${verrev_validate_args} && $luarocks lint ./validate-args-${verrev_validate_args}.rockspec && rm ./validate-args-${verrev_validate_args}.rockspec; }
 fail_lint_type_mismatch_string() { $luarocks lint "$testing_dir/testfiles/type_mismatch_string-1.0-1.rockspec"; }
@@ -373,10 +425,10 @@ fail_lint_type_mismatch_table() { $luarocks lint "$testing_dir/testfiles/type_mi
 test_list() { $luarocks list; }
 test_list_porcelain() { $luarocks list --porcelain; }
 
-test_make_with_rockspec() { rm -rf ./luasocket-${verrev_luasocket} && $luarocks download --src luasocket && $luarocks unpack ./luasocket-${verrev_luasocket}.src.rock && cd luasocket-${verrev_luasocket}/${srcdir_luasocket}  && $luarocks make luasocket-${verrev_luasocket}.rockspec && cd ../.. && rm -rf ./luasocket-${verrev_luasocket}; }
-test_make_default_rockspec() { rm -rf ./lxsh-${verrev_lxsh} &&  $luarocks download --src lxsh ${verrev_lxsh} &&  $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock &&  cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  &&  $luarocks make && cd ../.. && rm -rf ./lxsh-${verrev_lxsh}; }
-test_make_pack_binary_rock() { rm -rf ./lxsh-${verrev_lxsh} &&  $luarocks download --src lxsh ${verrev_lxsh} &&  $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock &&  cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  &&  $luarocks make --deps-mode=none --pack-binary-rock &&  [ -e ./lxsh-${verrev_lxsh}.all.rock ] &&  cd ../.. && rm -rf ./lxsh-${verrev_lxsh}; }
-fail_make_which_rockspec() { rm -rf ./luasocket-${verrev_luasocket} && $luarocks download --src luasocket && $luarocks unpack ./luasocket-${verrev_luasocket}.src.rock && cd luasocket-${verrev_luasocket}/${srcdir_luasocket}  && $luarocks make && cd ../.. && rm -rf ./luasocket-${verrev_luasocket}; }
+test_make_with_rockspec() { rm -rf ./luasocket-${verrev_luasocket} && $luarocks download --source luasocket && $luarocks unpack ./luasocket-${verrev_luasocket}.src.rock && cd luasocket-${verrev_luasocket}/${srcdir_luasocket}  && $luarocks make luasocket-${verrev_luasocket}.rockspec && cd ../.. && rm -rf ./luasocket-${verrev_luasocket}; }
+test_make_default_rockspec() { rm -rf ./lxsh-${verrev_lxsh} &&  $luarocks download --source lxsh ${verrev_lxsh} &&  $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock &&  cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  &&  $luarocks make && cd ../.. && rm -rf ./lxsh-${verrev_lxsh}; }
+test_make_pack_binary_rock() { rm -rf ./lxsh-${verrev_lxsh} &&  $luarocks download --source lxsh ${verrev_lxsh} &&  $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock &&  cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  &&  $luarocks make --deps-mode=none --pack-binary-rock &&  [ -e ./lxsh-${verrev_lxsh}.all.rock ] &&  cd ../.. && rm -rf ./lxsh-${verrev_lxsh}; }
+fail_make_which_rockspec() { rm -rf ./luasocket-${verrev_luasocket} && $luarocks download --source luasocket && $luarocks unpack ./luasocket-${verrev_luasocket}.src.rock && cd luasocket-${verrev_luasocket}/${srcdir_luasocket}  && $luarocks make && cd ../.. && rm -rf ./luasocket-${verrev_luasocket}; }
 
 test_new_version() { $luarocks download --rockspec luacov ${version_luacov} &&  $luarocks new_version ./luacov-${version_luacov}-1.rockspec 0.2 && rm ./luacov-0.*; }
 test_new_version_url() { $luarocks download --rockspec abelhas 1.0 && $luarocks new_version ./abelhas-1.0-1.rockspec 1.1 https://github.com/downloads/ittner/abelhas/abelhas-1.1.tar.gz && rm ./abelhas-*; }
@@ -392,9 +444,10 @@ test_path_lr_bin() { $luarocks path --lr-bin; }
 fail_purge_missing_tree() { $luarocks purge --tree="$testing_tree"; }
 test_purge() { $luarocks purge --tree="$testing_sys_tree"; }
 
-test_remove() { $luarocks build luacov ${version_luacov} && $luarocks remove luacov ${version_luacov}; }
+test_remove() { $luarocks build abelhas ${version_abelhas} && $luarocks remove abelhas ${version_abelhas}; }
 test_remove_force() { need_luasocket; $luarocks build lualogging && $luarocks remove --force luasocket; }
 fail_remove_deps() { need_luasocket; $luarocks build lualogging && $luarocks remove luasocket; }
+fail_remove_missing() { $luarocks remove missing_rock; }
 fail_remove_invalid_name() { $luarocks remove invalid.rock; }
 
 test_search_found() { $luarocks search zlib; }
@@ -402,24 +455,30 @@ test_search_missing() { $luarocks search missing_rock; }
 
 test_show() { $luarocks show luacov; }
 test_show_modules() { $luarocks show --modules luacov; }
+test_show_home() { $luarocks show --home luacov; }
 test_show_depends() { need_luasocket; $luarocks install $luasec && $luarocks show luasec; }
 test_show_oldversion() { $luarocks install luacov ${version_luacov} && $luarocks show luacov ${version_luacov}; }
 
 test_unpack_download() { rm -rf ./cprint-${verrev_cprint} && $luarocks unpack cprint && rm -rf ./cprint-${verrev_cprint}; }
-test_unpack_src() { rm -rf ./cprint-${verrev_cprint} && $luarocks download --src cprint && $luarocks unpack ./cprint-${verrev_cprint}.src.rock && rm -rf ./cprint-${verrev_cprint}; }
+test_unpack_src() { rm -rf ./cprint-${verrev_cprint} && $luarocks download --source cprint && $luarocks unpack ./cprint-${verrev_cprint}.src.rock && rm -rf ./cprint-${verrev_cprint}; }
 test_unpack_rockspec() { rm -rf ./cprint-${verrev_cprint} && $luarocks download --rockspec cprint && $luarocks unpack ./cprint-${verrev_cprint}.rockspec && rm -rf ./cprint-${verrev_cprint}; }
 test_unpack_binary() { rm -rf ./cprint-${verrev_cprint} && $luarocks build cprint && $luarocks pack cprint && $luarocks unpack ./cprint-${verrev_cprint}.${platform}.rock && rm -rf ./cprint-${verrev_cprint}; }
+fail_unpack_invalidpatch() { need_luasocket; $luarocks unpack "$testing_dir/testfiles/invalid_patch-0.1-1.rockspec"; }
+fail_unpack_invalidrockspec() { need_luasocket; $luarocks unpack "invalid.rockspec"; }
+
+fail_upload_invalidrockspec() { $luarocks upload "invalid.rockspec"; }
+fail_upload_invalidkey() { $luarocks upload --api-key="invalid" "invalid.rockspec"; }
 
 test_admin_help() { $luarocks_admin help; }
 
 test_admin_make_manifest() { $luarocks_admin make_manifest; }
-test_admin_add_rsync() { if [ "$travis" ]; then return; fi; $luarocks_admin --server=testing add "$testing_server/luasocket-${verrev_luasocket}.src.rock"; }
-test_admin_add_sftp() { if [ "$travis" ]; then return; fi; export LUAROCKS_CONFIG="$testing_dir/testing_config_sftp.lua" && $luarocks_admin --server=testing add ./luasocket-${verrev_luasocket}.src.rock; export LUAROCKS_CONFIG="$testing_dir/testing_config.lua"; }
+test_admin_add_rsync() { $luarocks_admin --server=testing add "$testing_server/luasocket-${verrev_luasocket}.src.rock"; }
+test_admin_add_sftp() { export LUAROCKS_CONFIG="$testing_dir/testing_config_sftp.lua" && $luarocks_admin --server=testing add ./luasocket-${verrev_luasocket}.src.rock; export LUAROCKS_CONFIG="$testing_dir/testing_config.lua"; }
 fail_admin_add_missing() { $luarocks_admin --server=testing add; }
 fail_admin_invalidserver() { $luarocks_admin --server=invalid add "$testing_server/luasocket-${verrev_luasocket}.src.rock"; }
-fail_admin_invalidrock() { if [ "$travis" ]; then return 1; fi; $luarocks_admin --server=testing add invalid; }
-test_admin_refresh_cache() { if [ "$travis" ]; then return; fi; $luarocks_admin --server=testing refresh_cache; }
-test_admin_remove() { if [ "$travis" ]; then return; fi; $luarocks_admin --server=testing remove luasocket-${verrev_luasocket}.src.rock; }
+fail_admin_invalidrock() { $luarocks_admin --server=testing add invalid; }
+test_admin_refresh_cache() { $luarocks_admin --server=testing refresh_cache; }
+test_admin_remove() { $luarocks_admin --server=testing remove luasocket-${verrev_luasocket}.src.rock; }
 fail_admin_remove_missing() { $luarocks_admin --server=testing remove; }
 
 fail_deps_mode_invalid_arg() { $luarocks remove luacov --deps-mode; }
@@ -429,8 +488,8 @@ test_deps_mode_order_sys() { $luarocks build --tree="$testing_tree" lpeg && $lua
 test_deps_mode_all_sys() { $luarocks build --tree="$testing_tree" lpeg && $luarocks build --deps-mode=all --tree="$testing_sys_tree" lxsh && [ `$luarocks_noecho list --tree="$testing_sys_tree" --porcelain lpeg | wc -l` = 0 ]; }
 test_deps_mode_none() { $luarocks build --tree="$testing_tree" --deps-mode=none lxsh; [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 0 ]; }
 test_deps_mode_nodeps_alias() { $luarocks build --tree="$testing_tree" --nodeps lxsh; [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 0 ]; }
-test_deps_mode_make_order() { $luarocks build --tree="$testing_sys_tree" lpeg && rm -rf ./lxsh-${verrev_lxsh} && $luarocks download --src lxsh ${verrev_lxsh} && $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock && cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  && $luarocks make --tree="$testing_tree" --deps-mode=order && cd ../.. && [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 0 ] && rm -rf ./lxsh-${verrev_lxsh}; }
-test_deps_mode_make_order_sys() { $luarocks build --tree="$testing_tree" lpeg && rm -rf ./lxsh-${verrev_lxsh} && $luarocks download --src lxsh ${verrev_lxsh} && $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock && cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  && $luarocks make --tree="$testing_sys_tree" --deps-mode=order && cd ../.. && [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 1 ] && rm -rf ./lxsh-${verrev_lxsh}; }
+test_deps_mode_make_order() { $luarocks build --tree="$testing_sys_tree" lpeg && rm -rf ./lxsh-${verrev_lxsh} && $luarocks download --source lxsh ${verrev_lxsh} && $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock && cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  && $luarocks make --tree="$testing_tree" --deps-mode=order && cd ../.. && [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 0 ] && rm -rf ./lxsh-${verrev_lxsh}; }
+test_deps_mode_make_order_sys() { $luarocks build --tree="$testing_tree" lpeg && rm -rf ./lxsh-${verrev_lxsh} && $luarocks download --source lxsh ${verrev_lxsh} && $luarocks unpack ./lxsh-${verrev_lxsh}.src.rock && cd lxsh-${verrev_lxsh}/lxsh-${version_lxsh}-1  && $luarocks make --tree="$testing_sys_tree" --deps-mode=order && cd ../.. && [ `$luarocks_noecho list --tree="$testing_tree" --porcelain lpeg | wc -l` = 1 ] && rm -rf ./lxsh-${verrev_lxsh}; }
 
 test_write_rockspec() { $luarocks write_rockspec git://github.com/keplerproject/luarocks; }
 test_write_rockspec_lib() { $luarocks write_rockspec git://github.com/mbalmer/luafcgi --lib=fcgi --license="3-clause BSD" --lua-version=5.1,5.2; }
@@ -452,8 +511,13 @@ run_tests() {
       echo "-------------------------------------------"
       reset_environment
       if $test
-      then echo "OK: Expected success."
-      else echo "FAIL: Unexpected failure."; exit 1
+      then
+         echo "OK: Expected success."
+      else
+         if [ $? = 99 ]
+         then echo "FAIL: Unexpected crash!"; exit 99
+         fi
+         echo "FAIL: Unexpected failure."; exit 1
       fi
    done
    grep "^fail_$1.*(" < $testing_dir/testing.sh | cut -d'(' -f1 | while read test
@@ -464,7 +528,11 @@ run_tests() {
       reset_environment
       if $test
       then echo "FAIL: Unexpected success."; exit 1
-      else echo "OK: Expected failure."
+      else
+         if [ $? = 99 ]
+         then echo "FAIL: Unexpected crash!"; exit 99
+         fi
+         echo "OK: Expected failure."
       fi
    done
 }
@@ -481,7 +549,11 @@ run_with_full_environment() {
    echo "==========================================="
    echo "Running with full environment"
    echo "==========================================="
-   build_environment luacov luafilesystem luasocket luabitop luaposix md5 lzlib
+   
+   local bitop=
+   [ "$luaversion" = "5.1.5" ] && bitop=luabitop
+   
+   build_environment luacov luafilesystem luasocket $bitop luaposix md5 lzlib
    run_tests $1
 }
 
@@ -493,12 +565,16 @@ run_all_tests() {
 run_all_tests $1
 #run_with_minimal_environment $1
 
-$testing_sys_tree/bin/luacov -c $testing_dir/luacov.config src/luarocks src/bin
-
 if [ "$travis" ]
 then
+   if [ "$TRAVIS" ]
+   then
+      build_environment luacov luafilesystem luacov-coveralls
+      ( cd $testing_dir; $testing_sys_tree/bin/luacov-coveralls || echo "ok" )
+   fi
+   $testing_sys_tree/bin/luacov -c $testing_dir/luacov.config src/luarocks src/bin
    grep "Summary" -B1 -A1000 $testing_dir/luacov.report.out
 else
+   $testing_sys_tree/bin/luacov -c $testing_dir/luacov.config src/luarocks src/bin
    cat "$testing_dir/luacov.report.out"
 fi
-
